package edu.umd.dawn.common.annotations;

import lombok.extern.log4j.Log4j2;
import org.apache.logging.log4j.Level;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.springframework.stereotype.Component;

/**
 * Traces entry and exit from functions at specified log level
 */
@Log4j2
@Aspect
@Component
public class TraceableAspect extends AspectBase<Traceable> {

    @Around("@annotation(Traceable)")
    public Object run(ProceedingJoinPoint jointPoint) throws Exception, Throwable {
        Traceable traceable = grabAnnotation(jointPoint, Traceable.class);

        String descriptor = getFullDescriptor(jointPoint);

        log.log(
                Level.forName(traceable.level().name(), traceable.level().intLevel()),
                String.format("entering %s", descriptor));
        Object proceed = jointPoint.proceed();
        log.info(String.format("exiting %s", descriptor));
        return proceed;
    }
}
