package edu.umd.dawn.common.exceptions;

import lombok.Getter;

@Getter
public class DawnException extends RuntimeException {

    private int statusCode;
    private String description;
    private String errorCode;
    private Exception base;

    public DawnException(DawnExceptionParameters parameters) {
        super(String.format("%s -- %s", parameters.getErrorCode(), parameters.getDescription()));
        statusCode = parameters.getStatusCode();
        description = parameters.getDescription();
        errorCode = parameters.getErrorCode();
    }

    public DawnException(DawnExceptionParameters parameters, Exception baseException) {
        this(parameters);
        base = baseException;
    }

    public DawnException(DawnExceptionParameters parameters, Exception baseException, String description) {
        this(parameters);
        base = baseException;
        this.description = description;
    }

    @Override
    public String getMessage() {
        return description;
    }

    @Override
    public StackTraceElement[] getStackTrace() {
        if (base != null) {
            return base.getStackTrace();
        }
        return super.getStackTrace();
    }

    public void setDescription(String description) {
        this.description = description;
    }
}
